/*
 * Copyright (C) 2012 by I.Kato @ BS21 Lab
 * DIPSW8x8 library for Arduino
 *    ver.1.00 (3/oct/2012)
 *
 * 対象     : 8 DIP SW シールド (BS21LAB-001)
 * 接続方法 : SPI 通信方式
 *
 * 注記1 : "8 DIP SW シールド" とは、8bit の DIP スイッチが８個有るシールドです。
 *         8bit×8 で、計 64bit 指定できます。それを SPI 通信にて読み取ります。
 *
 * 注記2 : 各 DIPSW の現在値の読み取りアルゴリズムの簡単な解説
 *     1. 読み取りたい DIPSW 番号のビット列を、MOSI ピンを通じて HC164 のシフトレジスタへセットします。
 *     2. SS ピンを Lowアクティブにすると、先ほど選択した DIPSW の現在の bit 状態が
 *        HC165 のシフトレジスタにラッチされます。
 *     3. その状態で SCK よりクロックパルスを送ると、HC165 のシフトレジスタの内容が、
 *        HC368 の３ステートインバータを通して、MISO ピンに出力されます。
 *     4. 8クロック送ると、8bit 分が読み出せる事になります。
 *     5. SS ピンを開放(非アクティブ)すると、HC165 のシフトレジスタはアンラッチされます。
 *
 * 詳細は http://www.bs21-lab.com/products/p001-8DIPSW/ を御覧下さい。
 *
 * This file is free software; you can redistribute it and/or modify it
 * under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1,
 * both as published by the Free Software Foundation.
 */

#include <SPI.h> // 必須です。
#include <DIPSW8x8.h>


// コンストラクタ：スレーブピン番号を指定
DIPSW8x8::DIPSW8x8(uint8_t slaveSelPin)
{
  _slaveSelPin = slaveSelPin; // private 変数に保持
}


// デストラクタ：特に何もしません
DIPSW8x8::~DIPSW8x8()
{
}


// 初期化処理：最初にこれを呼び出してください。
void DIPSW8x8::begin(void)
{
  pinMode(_slaveSelPin, OUTPUT); // 出力ポートに設定します。
  digitalWrite(_slaveSelPin, HIGH); // slaveSelPin を非アクティブにします。
}


// 終了処理：通常は呼び出す事は無いと思いますが…
void DIPSW8x8::end(void)
{
  digitalWrite(_slaveSelPin, HIGH); // slaveSelPin を非アクティブにします。
}


// SPIのモードを、8 DIPSW シールド用にセットします。
// 注記: 8 DIP SWシールドは、SPI ライブラリのデフォルト設定で呼び出す必要があります。
//       これらのパラメータを変更した場合は、この関数を呼び出す必要があります。
void DIPSW8x8::setReadMode(void)
{
  SPI.setBitOrder(MSBFIRST);  // 必須設定。SPI ライブラリのデフォルト設定にする事。
  SPI.setDataMode(SPI_MODE0); // 同上。

  // 以下の設定は変更しなくても通信速度が変わるだけで SPI 通信自体は大丈夫だと思われますが、
  // 挙動がおかしい場合は、以下も有効にしてみてください。
//  SPI.setClockDivider(SPI_CLOCK_DIV4);
}


// 指定の DIPSW 番号の現在の値を読み取ります。上記のアルゴリズム通りのプログラムです。
byte DIPSW8x8::readByte(byte swNo)
{
  byte cond; // 結果を格納する変数

  swNo &= B00000111; // DIPSW 番号を下位 3bit だけにします。上位 5bit は無視されます。

  // 1bit だけ LOW になった HC164 シフトレジスタ用のビット列を生成します。例：B11110111 という感じ
  byte bitOrder = ~(0x01 << swNo);

  // HC164 のシフトレジスタに上記のビット列をセットする事で、８つの DIPSW の内の１つだけを選択します。
  // なお、SS の ON/OFF に関わらず、HC164 は常にビット列を受け取りますので、注意してください。
  SPI.transfer(bitOrder);
  delayMicroseconds(50); // HC164 と DIPSW マトリックス回路の安定待ち

  // SS をアクティブにする事で、先ほど選択された DIPSW の現在の値が HC165 のシフトレジスタにラッチされます。
  // 同時に HC368 の３ステートインバータが有効になる事で MISO がオープンになります。
  digitalWrite(_slaveSelPin, LOW);
  delayMicroseconds(50); // 安定待ち

  // HC165 のシフトレジスタの値を MISO を経由して 1bit ずつシフトアウトします。これを８回繰り返します。
  // なお、ここの引数は読み取り結果に影響しませんので、何でも構いません。
  cond = SPI.transfer(bitOrder);
  delayMicroseconds(50); // 安定待ち

#if 0
  // ハードウエアの工夫により、HC165 は QH 出力を SI 入力に入れ直しています。
  // よって HC165 のシフトレジスタは何度読み出しても同じ値が読み出せるはずです。その確認の為のテストコードです。
  // 通常は１回読み出せば十分なので、この部分は無効にしてあります。
  cond = SPI.transfer(bitOrder);
  delayMicroseconds(50); // 安定待ち
#endif

  // SS を非アクティブにすると、HC386 の３ステートインバータが無効になり、MISO がクローズされます。
  // 同時に HC165 のシフトレジスタのラッチも解除(アンラッチ)されます。
  digitalWrite(_slaveSelPin, HIGH);

  return cond; // 読み取った値を返します。
}

