/*

  DIPSW8x8 Library - Udp NTP Client with DIPSW8x8's shield

 Get the time from a Network Time Protocol (NTP) time server
 Demonstrates use of UDP sendPacket and ReceivePacket 
 For more on NTP time servers and the messages needed to communicate with them, 
 see http://en.wikipedia.org/wiki/Network_Time_Protocol

 created 4 Sep 2010 
 by Michael Margolis
 modified 9 Apr 2012
 by Tom Igoe

  本スケッチは Arduino IDE v1.01 添付の Ethernet ライブラリの UdpNtpClient.ino を
  "8 DIP SW シールド"用に修正したスケッチ例です。
  8 DIP SW シールドの SW1～SW6 の値を、MAC アドレスとして Ethernet ライブラリに設定します。
  なお SW7～SW8 はフリー設定なので、その読み値をテキストにして、シリアルモニターに表示します。

  Circuit:
    pins 11(MOSI), 12(MISO), 13(SCK) for SPI communication in Arduino standard.
    pins 51(MOSI), 50(MISO), 52(SCK) for SPI communication in Arduino Mega.
    ICSP ピンからの SPI 接続もOK です。
    pin 9 for Slave Select for Your DIPSW8x8's shield.

  modified 3/oct/2012 by I.Kato @ BS21 Lab

 This code is in the public domain.

 */

#include <Ethernet.h>
#include <EthernetUdp.h>

// DIPSW8x8 ライブラリよりも先に SPI ライブラリを include する必要があります。
#include <SPI.h>

// DIPSW8x8 ライブラリを include します。
#include <DIPSW8x8.h>


// 御使用の 8 DIP SW シールドの SS ピン番号に合わせて、以下を修正してください。
// 注意：10番ピンは Ethernet シールドが使用していますので、それ以外のピンを割り当ててください。
DIPSW8x8 dipsw8x8(9); // DIPSW8x8 のインスタンスを作成します。


// 元の UdpNtpClient.ino では、以下の MAC アドレスを、手作業で書き換える必要がありましたが
// 8 DIP SW シールドの SW1～SW6 を読み込んで、プログラム的に書き換えるようにしました。
byte mac[] = {
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF }; // ここを手作業で書き換える必要はありません。

unsigned int localPort = 8888;      // local port to listen for UDP packets

IPAddress timeServer(192, 43, 244, 18); // time.nist.gov NTP server

const int NTP_PACKET_SIZE= 48; // NTP time stamp is in the first 48 bytes of the message

byte packetBuffer[ NTP_PACKET_SIZE]; //buffer to hold incoming and outgoing packets

// A UDP instance to let us send and receive packets over UDP
EthernetUDP Udp;

void setup()
{
 // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  SPI.begin(); // 最初に SPI ライブラリを初期化させてください。
  Serial.println("SPI.begin() done.");

  dipsw8x8.begin(); // 続いて 8DIPSW ライブラリを初期化させます。
  dipsw8x8.setReadMode(); // 8DIPSW 用の SPI モードを設定します。
  Serial.println("dipsw8x8.begin() done.");

  // 8 DIP SW シールドの SW1～SW6 を読み取って MAC アドレスとして設定し、
  // その値をテキストにして、シリアルモニターに転送します。
  Serial.print("My MAC addr : ");
  char str[8]; // テキスト変換用バッファ
  int swNo;
  for (swNo = DIPSW8x8_SW1; swNo <= DIPSW8x8_SW6; swNo++) { // SW1～SW6 までをループさせます
    mac[swNo] = dipsw8x8.readByte(swNo); // DIPSW の現在の値を読み取り、mac[] に設定します。

    sprintf(str, "%02X", mac[swNo]); // 読み取った値をテキストに変換
    Serial.print(str); // シリアルモニターに転送

    if (swNo < DIPSW8x8_SW6) { // SW6 以外の時のみ
      Serial.print('-'); // 分離記号(ハイフン)を表示
    }
  }
  Serial.print("\n\n");

//  delay(1000);

  // start Ethernet and UDP
  if (Ethernet.begin(mac) == 0) {
    Serial.println("Failed to configure Ethernet using DHCP");
    // no point in carrying on, so do nothing forevermore:
    for(;;) {
      ;
    }
  }
  Udp.begin(localPort);
}

void loop()
{
  sendNTPpacket(timeServer); // send an NTP packet to a time server

  // wait to see if a reply is available
  delay(1000);

  if ( Udp.parsePacket() ) {
    // We've received a packet, read the data from it
    Udp.read(packetBuffer,NTP_PACKET_SIZE);  // read the packet into the buffer

    //the timestamp starts at byte 40 of the received packet and is four bytes,
    // or two words, long. First, esxtract the two words:

    unsigned long highWord = word(packetBuffer[40], packetBuffer[41]);
    unsigned long lowWord  = word(packetBuffer[42], packetBuffer[43]);
    // combine the four bytes (two words) into a long integer
    // this is NTP time (seconds since Jan 1 1900):
    unsigned long secsSince1900 = (highWord << 16) | lowWord;
    Serial.print("Seconds since Jan 1 1900 = " );
    Serial.println(secsSince1900);

    // now convert NTP time into everyday time:
    Serial.print("Unix time = ");
    // Unix time starts on Jan 1 1970. In seconds, that's 2208988800:
    const unsigned long seventyYears = 2208988800UL;
    // subtract seventy years:
    unsigned long epoch = secsSince1900 - seventyYears;
    // print Unix time:
    Serial.println(epoch);

    // print the hour, minute and second:
    Serial.print("The UTC time is ");       // UTC is the time at Greenwich Meridian (GMT)
    Serial.print((epoch  % 86400L) / 3600); // print the hour (86400 equals secs per day)
    Serial.print(':');  
    if ( ((epoch % 3600) / 60) < 10 ) {
      // In the first 10 minutes of each hour, we'll want a leading '0'
      Serial.print('0');
    }
    Serial.print((epoch  % 3600) / 60); // print the minute (3600 equals secs per minute)
    Serial.print(':'); 
    if ( (epoch % 60) < 10 ) {
      // In the first 10 seconds of each minute, we'll want a leading '0'
      Serial.print('0');
    }
    Serial.println(epoch %60); // print the second


    // DIP SW7～8 の値を表示
    Serial.print("DIPSW7&8 status : ");
    char str[8]; // テキスト変換用バッファ
    int swNo;
    for (swNo = DIPSW8x8_SW7; swNo <= DIPSW8x8_SW8; swNo++) { // SW7～SW8 までをループさせます
      byte val = dipsw8x8.readByte(swNo); // 指定の DIPSW の現在の値を読み取ります。

      sprintf(str, "%02X ", val); // 読み取った値をテキストに変換
      Serial.print(str); // シリアルモニターに転送
    }
    Serial.print("\n\n");

  }
  // wait ten seconds before asking for the time again
  delay(10000);
}

// send an NTP request to the time server at the given address
unsigned long sendNTPpacket(IPAddress& address)
{
  // set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE); 
  // Initialize values needed to form NTP request
  // (see URL above for details on the packets)
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12]  = 49; 
  packetBuffer[13]  = 0x4E;
  packetBuffer[14]  = 49;
  packetBuffer[15]  = 52;

  // all NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  Udp.beginPacket(address, 123); //NTP requests are to port 123
  Udp.write(packetBuffer,NTP_PACKET_SIZE);
  Udp.endPacket(); 
}

